Attribute VB_Name = "modLargeAddressAware"
Option Explicit

Private Const EXE_FILE_LARGE_ADDRESS_AWARE As Integer = &H20

Private Const adModeReadWrite As Long = 3
Private Const adTypeBinary    As Long = 1
Private Const adSaveCreateOverWrite   As Long = 2

Public Enum LaaMode
    DisplayLaaStatusOnly
    TurnOnLaa
    TurnOffLaa
End Enum

Public Sub SetLaaFlag(ByVal ExeFileName As String, ByVal operationMode As LaaMode)

    Dim fileStream As Object
    Set fileStream = CreateObject("ADODB.Stream")
    fileStream.Type = adTypeBinary
    fileStream.Mode = adModeReadWrite
    fileStream.Open
    fileStream.LoadFromFile ExeFileName

    Dim LaaPosition As Integer
    LaaPosition = GetLaaPosition(fileStream)

    ' reading the byte with the LAA-Flag
    Dim readBuffer() As Byte
    fileStream.Position = LaaPosition
    readBuffer = fileStream.Read(1)

    ' Check if LAA flag set
    Dim LaaIsEnabled As Boolean
    LaaIsEnabled = CBool((readBuffer(0) And EXE_FILE_LARGE_ADDRESS_AWARE) = EXE_FILE_LARGE_ADDRESS_AWARE)
    
    Debug.Print "LAA is " & IIf(LaaIsEnabled, "", "NOT ") & "enabled."
    
    If Not operationMode = DisplayLaaStatusOnly Then
        Dim writeBuffer(0) As Byte
        Dim doWrite As Boolean
        
        ' Applying the desired LAA flag value
        If LaaIsEnabled _
            And operationMode = TurnOffLaa _
        Then
            writeBuffer(0) = (readBuffer(0) Xor EXE_FILE_LARGE_ADDRESS_AWARE)
            doWrite = True
        ElseIf Not LaaIsEnabled _
            And operationMode = TurnOnLaa _
        Then
            writeBuffer(0) = (readBuffer(0) Or EXE_FILE_LARGE_ADDRESS_AWARE)
            doWrite = True
        Else
            Debug.Print "Doing nothing"
        End If
        
        If doWrite Then
            Debug.Print "Switching "; IIf(operationMode = TurnOnLaa, "ON", "OFF") & " LAA"
            ' Writing LAA byte to stream
            fileStream.Position = LaaPosition
            fileStream.Write writeBuffer
    
            ' Saving stream to disk
            fileStream.SaveToFile ExeFileName, adSaveCreateOverWrite
            Debug.Print "Success - Updated file was written to disc"
        End If
    End If
    
    fileStream.Close

End Sub

Function GetLaaPosition(fileStream As Object) As Long

    Const MZ As Integer = 23117
    Const PE As Integer = 17744

    Const POSITION_PE_LOCATION = &H3C
    Const LAA_OFFSET_FROM_PE = 22

    Dim readBuffer() As Byte

    ' Reading "MZ" flag of .exe header
    readBuffer = fileStream.Read(2)
    If Not BytesToInteger(readBuffer(0), readBuffer(1)) = MZ Then
        Err.Raise 9999, "GetLaaPosition", "File is not a valid .exe file"
    End If

    ' reading position of PE header
    fileStream.Position = POSITION_PE_LOCATION
    readBuffer = fileStream.Read(2)

    ' reading "PE" flag of PE header
    Dim PeOffset As Integer
    PeOffset = BytesToInteger(readBuffer(0), readBuffer(1))
    fileStream.Position = PeOffset
    readBuffer = fileStream.Read(2)
    If Not BytesToInteger(readBuffer(0), readBuffer(1)) = PE Then
        Err.Raise 9999, "GetLaaPosition", "File is not a valid PE file"
    End If

    ' Position of LAA-Flag
    GetLaaPosition = PeOffset + LAA_OFFSET_FROM_PE

End Function

Public Function BytesToInteger(LoByte As Byte, HiByte As Byte) As Integer
    If HiByte And &H80 Then
        BytesToInteger = ((HiByte * &H100&) Or LoByte) Or &HFFFF0000
    Else
        BytesToInteger = (HiByte * &H100) Or LoByte
    End If
End Function



